import mimetypes
from pathlib import Path
from urllib.parse import urlparse

import weasyprint
from django.conf import settings
from django.contrib.staticfiles.finders import find
from django.core.files.storage import default_storage
from django.urls import get_script_prefix


def django_url_fetcher(url, *args, **kwargs):
    # load file:// paths directly from disk
    if url.startswith('file:'):
        mime_type, encoding = mimetypes.guess_type(url)
        url_path = urlparse(url).path
        data = {
            'mime_type': mime_type,
            'encoding': encoding,
            'filename': Path(url_path).name,
        }

        default_media_url = settings.MEDIA_URL in ('', get_script_prefix())
        if not default_media_url and url_path.startswith(settings.MEDIA_URL):
            media_root = settings.MEDIA_ROOT
            if isinstance(settings.MEDIA_ROOT, Path):
                media_root = f'{settings.MEDIA_ROOT}/'
            path = url_path.replace(settings.MEDIA_URL, media_root, 1)
            data['file_obj'] = default_storage.open(path)
            return data

        elif settings.STATIC_URL and url_path.startswith(settings.STATIC_URL):
            path = url_path.replace(settings.STATIC_URL, '', 1)
            data['file_obj'] = open(find(path), 'rb')
            return data

    # fall back to weasyprint default fetcher
    return weasyprint.default_url_fetcher(url, *args, **kwargs)



from pathlib import Path

import django
from django.conf import settings
from django.urls import path
from django.views.generic import TemplateView

from django_weasyprint import WeasyTemplateResponseMixin, WeasyTemplateView, views


class BaseView(TemplateView):
    # simple Django CBV with template
    template_name = 'example.html'


class PDFView(WeasyTemplateResponseMixin, BaseView):
    # just add mixin and set content-type
    content_type = views.CONTENT_TYPE_PDF
    # set filename for download
    pdf_filename = 'le-foo.pdf'


class PNGView(WeasyTemplateView):
    # png view from scratch
    template_name = 'example.html'
    content_type = views.CONTENT_TYPE_PNG


urlpatterns = [
    path('html/', BaseView.as_view()),
    path('pdf/', PDFView.as_view()),
    path('png/', PNGView.as_view()),
]


settings.configure(
    ALLOWED_HOSTS=['testserver'],
    ROOT_URLCONF=__name__,
    MIDDLEWARE=[],
    TEMPLATES=[{
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [Path(__file__).parent / 'templates'],
    }],
    DATABASES={'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }},
)
django.setup()



from django.test import SimpleTestCase

import django_weasyprint.tests  # noqa
from django_weasyprint.views import CONTENT_TYPE_PDF, CONTENT_TYPE_PNG


class WeasyTemplateViewTestCase(SimpleTestCase):
    def test_get_html(self):
        response = self.client.get('/html/')
        self.assertEqual(response.status_code, 200)

        self.assertEqual(sorted(response._headers.keys()), ['content-type'])
        self.assertEqual(response['content-type'], 'text/html; charset=utf-8')
        self.assertEqual(response.content, b'<h1>Example template</h1>\n')

    def test_get_pdf(self):
        response = self.client.get('/pdf/')
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            sorted(response._headers.keys()),
            ['content-disposition', 'content-type'],
        )
        self.assertEqual(response['content-type'], CONTENT_TYPE_PDF)
        self.assertEqual(
            response['content-disposition'],
            'attachment;filename="le-foo.pdf"',
        )
        self.assertEqual(response.content[:4], b'%PDF')

    def test_get_png(self):
        response = self.client.get('/png/')
        self.assertEqual(response.status_code, 200)

        self.assertEqual(sorted(response._headers.keys()), ['content-type'])
        self.assertEqual(response['content-type'], CONTENT_TYPE_PNG)
        self.assertEqual(response.content[1:4], b'PNG')
